import Logger, { LoggerType } from '#application/logger'
import { UUID } from '#application/types'
import { Character } from '#entities/character'
import SocketManager from '#managers/socketManager'
import MapManager from '#managers/mapManager'
import MapCharacter from '#models/mapCharacter'

interface TeleportOptions {
  targetMapId: UUID
  targetX: number
  targetY: number
  rotation?: number
  isInitialJoin?: boolean
  character?: Character
}

class TeleportService {
  private readonly logger = Logger.type(LoggerType.GAME)

  public async teleportCharacter(characterId: UUID, options: TeleportOptions): Promise<boolean> {
    const { targetMapId, targetX, targetY, rotation = 0, isInitialJoin = false, character } = options

    const socket = SocketManager.getSocketByCharacterId(characterId)
    const targetMap = MapManager.getMapById(targetMapId)

    if (!socket || !targetMap) {
      this.logger.error(`Teleport failed - Missing socket or target map for character ${characterId}`)
      return false
    }

    if (isInitialJoin && !character) {
      this.logger.error('Initial join requires character data')
      return false
    }

    const existingCharacter = !isInitialJoin && MapManager.getCharacterById(characterId)
    const mapCharacter = isInitialJoin
      ? new MapCharacter(character!)
      : existingCharacter ||
        (() => {
          this.logger.error(`Teleport failed - Character ${characterId} not found in MapManager`)
          return null
        })()

    if (!mapCharacter) return false

    try {
      const currentMapId = mapCharacter.character.map?.id
      const io = SocketManager.getIO()

      // Handle current map cleanup
      if (currentMapId) {
        socket.leave(currentMapId)
        MapManager.removeCharacter(characterId)
        io.in(currentMapId).emit('map:character:leave', characterId)
      }

      // Update character position and map
      await mapCharacter.character.setPositionX(targetX).setPositionY(targetY).setRotation(rotation).setMap(targetMap.getMap()).update()

      // Join new map
      socket.join(targetMapId)
      targetMap.addCharacter(mapCharacter.character)

      // Notify clients
      io.in(targetMapId).emit('map:character:join', mapCharacter)
      socket.emit('map:character:teleport', {
        map: targetMap.getMap(),
        characters: targetMap.getCharactersInMap()
      })

      return true
    } catch (error) {
      this.logger.error(`Teleport error for character ${characterId}: ${error}`)
      return false
    }
  }
}

export default new TeleportService()
